/* ***** BEGIN LICENSE BLOCK *****
   - Version: MPL 1.1/GPL 2.0/LGPL 2.1
   -
   - The contents of this file are subject to the Mozilla Public License Version
   - 1.1 (the "License"); you may not use this file except in compliance with
   - the License. You may obtain a copy of the License at
   - http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS IS" basis,
   - WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
   - for the specific language governing rights and limitations under the
   - License.
   -
   - The Original Code is Bookmark Current Tab Set.
   -
   - The Initial Developer of the Original Code is Jake Kasprzak.
   - Portions created by the Initial Developer are Copyright (C) 2009
   - the Initial Developer. All Rights Reserved.
   -
   - Contributor(s):
   -  Jake Kasprzak <jake@kasprzak.ca>
   -
   - Alternatively, the contents of this file may be used under the terms of
   - either the GNU General Public License Version 2 or later (the "GPL"), or
   - the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
   - in which case the provisions of the GPL or the LGPL are applicable instead
   - of those above. If you wish to allow use of your version of this file only
   - under the terms of either the GPL or the LGPL, and not to allow others to
   - use your version of this file under the terms of the MPL, indicate your
   - decision by deleting the provisions above and replace them with the notice
   - and other provisions required by the LGPL or the GPL. If you do not delete
   - the provisions above, a recipient may use your version of this file under
   - the terms of any one of the MPL, the GPL or the LGPL.
   -
   - ***** END LICENSE BLOCK ***** */

//function to create namespaces in order to avoid interference with other add-ons
//refer to: http://blogger.ziesemer.com/2008/05/javascript-namespace-function.html   
String.prototype.namespace = function(separator) {
  var ns = this.split(separator || '.'),
    o = window,
    i,
    len;
  for (i = 0, len = ns.length; i < len; i++) {
    o = o[ns[i]] = o[ns[i]] || {};
  }
  return o;
};


//ca.kasprzak.jake.bcts_openDialog: simply opens a dialog for bookmarking all tabs
'ca.kasprzak.jake.bcts_openDialog'.namespace();
ca.kasprzak.jake.bcts_openDialog = function () { 

	window.openDialog('chrome://bcts/content/dialogs/bookmarkTabDialog.xul','bctsDialog', "centerscreen,chrome,dialog,resizable,modal");

}


//ca.kasprzak.jake.bcts_bookmark: the function for bookmarking all tabs. How the bookmarking is done depends on whether the quick bookmarking option was selected or not
'ca.kasprzak.jake.bcts_bookmark'.namespace();
ca.kasprzak.jake.bcts_bookmark = function (quickBookmark) {
		
	try {
		
		//variable for storing data in each tab
		var theBrowsers;
		
		//the page title and the URL of each tab
		var pageTitle = "";
		var url = "";
		
		//what the bookmark folder title is to be by default
		var dateString;
		
		//the folder to which a bookmark folder is to be added
		var folder;
		
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
		                    getService(Components.interfaces.nsIPrefService);
		prefs = prefs.getBranch("extensions.bcts.");
		
		var bookmarks = Components.classes["@mozilla.org/browser/nav-bookmarks-service;1"]
	                               .getService(Components.interfaces.nsINavBookmarksService);
		
		
		//retrieve information on tab contents, the location to which bookmarks are to be saved, and the name of the folder that will contain these bookmarks
		if (quickBookmark) {
			
			theBrowsers = gBrowser.browsers;
			var dateFormat = prefs.getCharPref("dateFormat");
			var includeTime = prefs.getBoolPref("includeTime");
			
			dateString = ca.kasprzak.jake.bcts_returnDate(dateFormat, includeTime);
			
			folder = bookmarks.bookmarksMenuFolder;
		
		}
		else {
		
			var gb = window.opener.getBrowser();
		
			theBrowsers = gb.browsers;
			
			dateString = document.getElementById("namePicker").value;
			var folderString = document.getElementById("folderPicker").value;
			
			switch (folderString) {
				case "bookmarks.bookmarksMenuFolder":
					folder = bookmarks.bookmarksMenuFolder;
					break;
				case "bookmarks.toolbarFolder":
					folder = bookmarks.toolbarFolder;
					break;
				default:
					folder = bookmarks.unfiledBookmarksFolder;
			}
			
		}
		
		var folderId = bookmarks.createFolder(
					folder, 
					dateString,
					bookmarks.DEFAULT_INDEX);
		
		//after creating this new bookmark folder, go through each tab and bookmark each one
		for (var i = 0; i < theBrowsers.length; i++) {
				
			var webNav = theBrowsers[i].webNavigation;
			
			var url = webNav.currentURI.spec;
			try {
				pageTitle = webNav.document.title  || url;
			} 
			catch (e) {
				pageTitle = url;
			}
			
			var bookmarkURI = Components.classes["@mozilla.org/network/io-service;1"]
			  .getService(Components.interfaces.nsIIOService)
			  .newURI(url, null, null);
			
			
			var bookmarkId = bookmarks.insertBookmark(
			  folderId, 
			  bookmarkURI,  
			  i, 
			  pageTitle);
			
		}
	}
	catch(err) { alert("Error occurred in extension:"+ err) }
}



//ca.kasprzak.jake.bcts_initMainDialog: this initializes the data that is to be initially in the text box when adding bookmarks
'ca.kasprzak.jake.bcts_initMainDialog'.namespace();
ca.kasprzak.jake.bcts_initMainDialog = function () { 

	var dateString;
	
	var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService);
	prefs = prefs.getBranch("extensions.bcts.");
	
	var bookmarks = Components.classes["@mozilla.org/browser/nav-bookmarks-service;1"]
                               .getService(Components.interfaces.nsINavBookmarksService);
	
	
	var dateFormat = prefs.getCharPref("dateFormat");
	var includeTime = prefs.getBoolPref("includeTime");
	
	dateString = ca.kasprzak.jake.bcts_returnDate(dateFormat, includeTime);
	
	document.getElementById("namePicker").removeAttribute('selectedItem');
	
	document.getElementById("namePicker").value = dateString;
	
	document.getElementById("namePicker").selectionStart = dateString.length; 
	document.getElementById("namePicker").selectionEnd = dateString.length;
	
	document.getElementById("namePicker").focus();
	
}


//ca.kasprzak.jake.bcts_returnDate: given information on what is set in the options, return the string that is to be the bookmark title 
'ca.kasprzak.jake.bcts_returnDate'.namespace();
ca.kasprzak.jake.bcts_returnDate = function (dateFormat, includeTime) { 
	
	var dateObj = new Date();
	
	//the string that will be returned by this function
	var dateString;
	
	var month = dateObj.getMonth() + 1;
	var defaultString = dateObj.toString().substr(0,15);
	
	switch (dateFormat) {
		
		case 'o':
			dateString = defaultString
			break;
		case 'm':
			dateString = month + "-" + dateObj.getDate() + "-" + dateObj.getFullYear();
			break;
		case 'd':
			dateString = dateObj.getDate() + "-" + month + "-" + dateObj.getFullYear();
			break;
		case 'y':
			dateString = dateObj.getFullYear() + "-" + month + "-" + dateObj.getDate();
			break;
		default:
			dateString = defaultString;
			break;
	}
	
	//add the time to the string if it is specified that it must be included, and ensure the format of the time is correct
	if (includeTime) {
		
		var hours;
		var minutes;
		var seconds;
		
		hours = (dateObj.getHours() >= 10) ? dateObj.getHours() : "0" + dateObj.getHours();
		minutes = (dateObj.getMinutes() >= 10) ? dateObj.getMinutes() : "0" + dateObj.getMinutes();
		seconds = (dateObj.getSeconds() >= 10) ? dateObj.getSeconds() : "0" + dateObj.getSeconds();
		
		dateString = dateString + " " + hours + ":" + minutes + ":" + seconds;
	}
	
	return dateString;
}